"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Parser = require("tree-sitter");
const highlight_tree_sitter_1 = require("highlight-tree-sitter");
function convertSexp(sexp, convertElement, convertText) {
    function print(node) {
        if (typeof node === 'string')
            return convertText(node);
        const [name, ...children] = node;
        return convertElement(name, children.map(print));
    }
    return print(sexp);
}
function highlightTree(scopeMappings, text, tree) {
    const full = highlight_tree_sitter_1.fullSexp(text, tree);
    const highlight = highlight_tree_sitter_1.highlightSexpFromScopes(full, scopeMappings);
    return convertSexp(highlight.sexp, (name, children) => {
        const className = name.split('.').slice(1);
        return {
            type: 'element',
            tagName: 'span',
            properties: {
                className
            },
            children
        };
    }, value => ({ type: 'text', value }));
}
exports.highlightTree = highlightTree;
function isParser(p) {
    return !!p.parse;
}
function highlightText(arg1, arg2, arg3) {
    const { parser, scopeMappings, text } = (() => {
        if (isParser(arg1)) {
            if (arg3)
                return { parser: arg1, scopeMappings: arg2, text: arg3 };
        }
        else if (arg1.grammar && arg1.scopeMappings && typeof arg2 === 'string') {
            const parser = new Parser();
            parser.setLanguage(arg1.grammar);
            return { parser, scopeMappings: arg1.scopeMappings, text: arg2 };
        }
        throw new Error('Invalid arguments to highlightText()');
    })();
    const tree = parser.parse(text);
    return highlightTree(scopeMappings, text, tree);
}
exports.highlightText = highlightText;
